package at.jku.ssw.pi.search.ue10;

import at.jku.ssw.pi.search.TextSearcher;

public class BruteForceSearcher extends TextSearcher {

	@Override
	public int countOccurences(String text, String pattern, boolean matchCase, boolean wholeWordOnly) {
		if (pattern.length() == 0) {
			throw new IllegalArgumentException("empty pattern");
		}
		if (!matchCase) {
			pattern = pattern.toLowerCase();
		}
		int count = 0;
		for (int i = 0; i <= text.length() - pattern.length(); i++) {
			if (!wholeWordOnly || isWholeWordAt(text, i, i + pattern.length() - 1)) {
				int k = 0;
				while (k < pattern.length() && matches(pattern.charAt(k), text.charAt(i + k), matchCase)) {
					k++;
				}
				if (k == pattern.length()) {
					count++;
				}
			}
		}
		return count;
	}

	private boolean matches(char patternCh, char textCh, boolean matchCase) {
		if (!matchCase) {
			assert Character.isLowerCase(patternCh);
			textCh = Character.toLowerCase(textCh);
		}
		return (patternCh == textCh);
	}

	boolean isWholeWordAt(String text, int firstIndex, int lastIndex) {
		if (firstIndex > 0 && Character.isLetterOrDigit(text.charAt(firstIndex - 1))) {
			return false;
		}
		if (lastIndex + 1 < text.length() && Character.isLetterOrDigit(text.charAt(lastIndex + 1))) {
			return false;
		}
		return true;
	}

}
